#!/usr/bin/env python
# -*- coding=utf-8 -*-

import neuron as nr
import dataframe as df

class NeuralNetwork(object):
    u""" Simple neural network 
         with 1 level of hidden neuron, and fully connected network """
    def __init__(self, nbIn, nbHidden, nbOut):
        self.neuronHidden = [ nr.Neuron(nbIn) for i in range(nbHidden) ]
        self.neuronOut = [ nr.Neuron(nbHidden) for i in range(nbOut) ]

    def predict(self, X):
        Xh = [ ]
        for n in self.neuronHidden:
            Xh.append( n.predict(X) )
        Xout = [ ]
        for n in self.neuronOut:
            Xout.append( n.predict(Xh) )
        return(Xout)

    def __str__(self):
        res = ""
        res += str(len(self.neuronHidden)) + "\n"
        for n in self.neuronHidden:
            res += str(n) + "\n"
        res += str(len(self.neuronOut))  + "\n"
        for n in self.neuronOut:
            res += str(n) + "\n"
        return(res) 

def distance2(u, v):
    u""" Distance between 2 vectors """
    d = 0
    for i in range(len(u)):
        d += (u[i] - v[i]) * (u[i] - v[i])
    return(d)

def mse(nn, dataframe):
    u""" Mean square error of the network on the dataframe """
    s = 0
    for d in dataframe.df:
        v = [ 0 ] * len(nn.neuronOut)
        v[d.y] = 1
        s += distance2( v, nn.predict(d.X) )
    return( float(s) / dataframe.size() )

def argmax(l):
    u""" Index of the maximum value of the list """
    imax = 0
    m = l[imax]
    for i in range(1,len(l)):
        if m < l[i]:
            m = l[i]
            imax = i
    return(imax)

def errorRate(nn, dataframe):
    u""" Error rate of the network on the data frame 
    (answer of network = index of the neuron with the largest value """
    s = 0
    for d in dataframe.df:
        out = argmax( nn.predict(d.X) )
        if out != d.y:
            s += 1
    return( float(s) / dataframe.size() )

if __name__ == "__main__":
    nn = NeuralNetwork(64, 4, 10)
    print nn
    print nn.predict( [ 1 ] * 64 )

    dataf = df.Dataframe("digits.csv")
    print mse(nn, dataf)
    print errorRate(nn, dataf)
