#!/usr/bin/env python
# -*- coding=utf-8 -*-

import random as rnd
import math

class Neuron(object):
    u""" Neuron design to be used in a learning process """

    def __init__(self, dim = 2):
        u""" Constructor """
        self.dim    = dim  # number of input of the neuron
        self.W      = [ ]  # Weights (first weight is the threshold)
        self.output = 0    # output of the neuron
        self.deltaW = [ ]  # learning: difference of weights
        self.delta  = 0    # learning: gradient
        self.init()

    def init(self):
        u""" Random initialization """
        self.W =  [ rnd.uniform(-1, 1) for i in range(self.dim + 1) ]
        self.output = 0
        # for the learning process
        self.delatW =  [ 0 ] * (self.dim + 1)
        self.delta  = 0
        
    def predict(self, X):
        u""" Compute the output of the neuron """
        # the first number is the threshold
        s = self.W[0]
        for i in range(1, len(self.W)):
            s += self.W[i] * X[i - 1]

        # Sigmoide activation function
        self.output = float(1) / ( 1 + math.exp(-s) )
        
        return self.output

    def updateW(self):
        u""" Update the weights according to the difference """
        for k in range(len(self.W)):
            self.W[k] += self.deltaW[k] 

    def __str__(self):
        u""" Print """
        res = ""
        for w in self.W:
            res += str(w) + " "
        res += "(%f)" % (self.output)
        return(res) 


if __name__ == "__main__":
    neuron = Neuron(3)
    print neuron
    print neuron.predict([ 1, 1, 1 ])
    print neuron
    
